<div id="icons_banner" class="flex-resource-banner">
    {% for custom_resource in data.custom_resources %}
        {% if custom_resource.display_icon %}
            {% if data.display_not_resource and not custom_resource.type == data.type_resource or custom_resource.type == data.type_resource %}
                <div id="cnt_{{custom_resource.slug}}" class="resource-container">
                    <i class="fas {{custom_resource.icon}} fa-5x" aria-hidden="true" style="color: {{custom_resource.icon_color}}"></i>
                    <p>{{custom_resource.title}}</p>
                    <div class="tick">
                         <i class="fas fa-check-circle-o fa-2x" aria-hidden="true"></i>
                    </div>
                </div>
            {% endif %}
        {% endif %}
    {% endfor %}
</div>
