{% extends "core_main_app/_render/admin/theme/tools/section.html" %}
{% load static %}

{% block section_title %} {{ data.object_name }}{% endblock %}

{% block section_content %}
<div class="row">
    <div class="col-md-12">
        {% include 'core_main_registry_app/admin/custom_registry/table.html' %}
    </div>
</div>
{% endblock %}